/*
  L(#)/M(#)-convexity test utility
 */

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <math.h>
#include <assert.h>
#include "odicon.h"
#include "con_util.h"

/* usefull macro in caller func */
#define TEST(dim,f) \
    convexity_print(dim,f,#f,lower,upper,sum)   \
    concavity_print(dim,f,#f,lower,upper,sum)
#define RANGE_PRINT(dim) range_print(dim,lower,upper,sum)

static void con_print(int dim, double f(int,int*), char *f_name,
                      int lower[], int upper[], int sum, char *con) {
    int ret;
    static char *yesno[] = { "NOT", "   " };

    counter_reset();
    ret = is_lgconv_gene(dim, f, lower, upper);
    counter_print();
    printf("%s(x) is %s an L#-%s func.\n", f_name, yesno[!!ret], con);

    counter_reset();
    ret = is_lconv_gene(dim, f, lower, upper);
    counter_print();
    printf("%s(x) is %s an  L-%s func.\n", f_name, yesno[!!ret], con);

    counter_reset();
    ret = is_mgconv_gene(dim, f, lower, upper);
    counter_print();
    printf("%s(x) is %s an M#-%s func.\n", f_name, yesno[!!ret], con);

    counter_reset();
    ret = is_mconv_gene(dim, f, lower, upper, sum);
    counter_print();
    printf("%s(x) is %s an  M-%s func.\n", f_name, yesno[!!ret], con);
/*
    counter_reset();
    ret = is_mconv_gene2(dim, f, lower, upper, sum);
    counter_print();
    printf("%s(x) is %s an  M-%s func.(2)\n", f_name, yesno[!!ret], con);
*/
    printf("\n");
}

void convexity_print(int dim, double f(int,int*), char *f_name,
                     int lower[], int upper[], int sum) {
    con_print(dim, f, f_name, lower, upper, sum, "convex");
}

static double (*stored_f)(int,int*);

/* negative_f(x) = -stored_f(x) */
static double negative_f(int dim, int x[]) {
    double r = stored_f(dim, x);
    if (r == PLUS_INFINITY) return PLUS_INFINITY;
    return -r;
}


void concavity_print(int dim, double f(int,int*), char *f_name,
                     int lower[], int upper[], int sum) {
    stored_f = f;
    con_print(dim, negative_f, f_name, lower, upper, sum, "concave");
}

void range_print(int dim, int *lower, int *upper, int sum) {
    printf("upper = "); ivector_print(dim, upper); printf("\n");
    printf("lower = "); ivector_print(dim, lower); printf("\n");
    printf("sum   = %d\n", sum);
}
